@@ -1,21 +1,20 @@
-/* gda-meta-struct.h
+/*
+ * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
  *
- * Copyright (C) 2008 - 2011 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #ifndef __GDA_META_STRUCT_H_
@@ -41,7 +40,8 @@ extern GQuark gda_meta_struct_error_quark (void);
 typedef enum {
 	GDA_META_STRUCT_UNKNOWN_OBJECT_ERROR,
         GDA_META_STRUCT_DUPLICATE_OBJECT_ERROR,
-        GDA_META_STRUCT_INCOHERENCE_ERROR
+        GDA_META_STRUCT_INCOHERENCE_ERROR,
+	GDA_META_STRUCT_XML_ERROR
 } GdaMetaStructError;
 
 
@@ -82,6 +82,15 @@ typedef enum {
 /*
  * Controls which features are computed about database objects
  */
+/**
+ * GdaMetaStructFeature:
+ * @GDA_META_STRUCT_FEATURE_NONE: database objects only have their own attributes
+ * @GDA_META_STRUCT_FEATURE_FOREIGN_KEYS: foreign keys are computed for tables
+ * @GDA_META_STRUCT_FEATURE_VIEW_DEPENDENCIES: for views, the tables they use are also computed
+ * @GDA_META_STRUCT_FEATURE_ALL: all the features are computed
+ *
+ * Controls which features are computed about database objects.
+ */
 typedef enum {
 	GDA_META_STRUCT_FEATURE_NONE              = 0,
 	GDA_META_STRUCT_FEATURE_FOREIGN_KEYS      = 1 << 0,
@@ -105,15 +114,19 @@ typedef enum {
 
 /**
  * GdaMetaTable:
- * @columns: list of #GdaMetaTableColumn structures, one for each column in the table
+ * @columns: (element-type Gda.MetaTableColumn): list of #GdaMetaTableColumn structures, one for each column in the table
  * @pk_cols_array: index of the columns part of the primary key for the table (WARNING: columns numbering
  *                 here start at 0)
  * @pk_cols_nb: size of the @pk_cols_array array
- * @reverse_fk_list: list of #GdaMetaTableForeignKey where the referenced table is this table
- * @fk_list: list of #GdaMetaTableForeignKey for this table
+ * @reverse_fk_list: (element-type Gda.MetaTableForeignKey): list of #GdaMetaTableForeignKey where the referenced table is this table
+ * @fk_list: (element-type Gda.MetaTableForeignKey): list of #GdaMetaTableForeignKey for this table
  *
  * This structure specifies a #GdaMetaDbObject to represent a table's specific attributes,
  * its contents must not be modified.
+ *
+ * Note that in some cases, the columns cannot be determined for views, and in this case the
+ * @columns will be %NULL (this can be the case for example with SQLite where a view
+ * uses a function which is not natively provided by SQLite.
  */
 typedef struct {
 	/*< public >*/
@@ -169,7 +182,7 @@ typedef struct {
  * @obj_short_name: the shortest way to name the object
  * @obj_full_name: the full name of the object (in the &lt;schema&gt;.&lt;nameagt; notation
  * @obj_owner: object's owner
- * @depend_list: list of #GdaMetaDbObject pointers on which this object depends (through foreign keys
+ * @depend_list: (element-type Gda.MetaDbObject): list of #GdaMetaDbObject pointers on which this object depends (through foreign keys
  *               or tables used for views)
  *
  * Struture to hold information about each database object (tables, views, ...),
@@ -243,7 +256,7 @@ typedef struct {
  * @gtype: the detected column's #GType
  * @pkey: tells if the column is part of a primary key
  * @nullok: tells if the column can be %NULL
- * @default_value: the column's default value
+ * @default_value: (allow-none): the column's default value, represented as a valid SQL value (surrounded by simple quotes for strings, ...), or %NULL if column has no default value
  *
  * This structure represents a table of view's column, its contents must not be modified.
  */
@@ -278,10 +291,10 @@ const GValue *gda_meta_table_column_get_attribute (GdaMetaTableColumn *tcol, con
 void          gda_meta_table_column_set_attribute (GdaMetaTableColumn *tcol, const gchar *attribute, const GValue *value,
 						   GDestroyNotify destroy);
 /**
- * gda_meta_table_column_set_attribute_static
+ * gda_meta_table_column_set_attribute_static:
  * @column: a #GdaMetaTableColumn
  * @attribute: attribute's name
- * @value: a #GValue, or %NULL
+ * @value: (allow-none): a #GValue, or %NULL
  *
  * This function is similar to gda_meta_table_column_set_attribute() but for static strings
  */
@@ -338,20 +351,27 @@ typedef struct {
 	gchar           **ref_pk_names_array; /* Ref PK fields names */
 
 	/*< private >*/
-	gpointer          on_update_policy; /* pointer containing the GdaMetaForeignKeyPolicy integer
+	GdaMetaForeignKeyPolicy on_update_policy; /* pointer containing the GdaMetaForeignKeyPolicy integer
 					     * to keep ABI from 4.0, use GINT_TO_POINTER and
 					     * GPOINTER_TO_INT */
-	gpointer          on_delete_policy; /* pointer containing the GdaMetaForeignKeyPolicy integer
+	GdaMetaForeignKeyPolicy on_delete_policy; /* pointer containing the GdaMetaForeignKeyPolicy integer
 					     * to keep ABI from 4.0, use GINT_TO_POINTER and
 					     * GPOINTER_TO_INT */
-	gpointer          declared; /* pointer to a boolean to keep ABI from 4.0.
+	gboolean          declared; /* pointer to a boolean to keep ABI from 4.0.
 				     * Any non NULL if FK has been declared in meta data */
 
 	/*< public >*/
 	gchar            *fk_name;
+
+	/*< private >*/
+	/* Padding for future expansion */
+	gpointer _gda_reserved1;
+	gpointer _gda_reserved2;
+	gpointer _gda_reserved3;
+	gpointer _gda_reserved4;
 } GdaMetaTableForeignKey;
 /**
- * GDA_META_TABLE_FOREIGN_KEY
+ * GDA_META_TABLE_FOREIGN_KEY:
  * @fk: a pointer
  *
  * Casts @fk to a #GdaMetaTableForeignKey (no check is actuelly being done on @fk's validity)
@@ -368,7 +388,7 @@ typedef struct {
  *
  * Returns: the policy as a #GdaMetaForeignKeyPolicy
  */
-#define GDA_META_TABLE_FOREIGN_KEY_ON_UPDATE_POLICY(fk) ((GdaMetaForeignKeyPolicy) GPOINTER_TO_INT ((GdaMetaTableForeignKey*)(fk)->on_update_policy))
+#define GDA_META_TABLE_FOREIGN_KEY_ON_UPDATE_POLICY(fk) (((GdaMetaTableForeignKey*)(fk))->on_update_policy)
 
 /**
  * GDA_META_TABLE_FOREIGN_KEY_ON_DELETE_POLICY:
@@ -378,10 +398,10 @@ typedef struct {
  *
  * Returns: the policy as a #GdaMetaForeignKeyPolicy
  */
-#define GDA_META_TABLE_FOREIGN_KEY_ON_DELETE_POLICY(fk) ((GdaMetaForeignKeyPolicy) GPOINTER_TO_INT ((GdaMetaTableForeignKey*)(fk)->on_delete_policy))
+#define GDA_META_TABLE_FOREIGN_KEY_ON_DELETE_POLICY(fk) (((GdaMetaTableForeignKey*)(fk))->on_delete_policy)
 
 /**
- * GDA_META_TABLE_FOREIGN_KEY_IS_DECLARED
+ * GDA_META_TABLE_FOREIGN_KEY_IS_DECLARED:
  * @fk: a pointer to a #GdaMetaTableForeignKey
  *
  * Tells if @fk is an actual foreign key defined in the database's schema, or if it is an indication which
@@ -389,7 +409,86 @@ typedef struct {
  *
  * Returns: %TRUE if @fk has been declared in the database's meta data and %FALSE if @fk is an actual foreign key defined in the database's schema
  */
-#define GDA_META_TABLE_FOREIGN_KEY_IS_DECLARED(fk) ((((GdaMetaTableForeignKey*)(fk))->declared) ? TRUE : FALSE)
+#define GDA_META_TABLE_FOREIGN_KEY_IS_DECLARED(fk) (((GdaMetaTableForeignKey*)(fk))->declared)
+
+/**
+ * SECTION:gda-meta-struct
+ * @short_description: In memory representation of some database objects
+ * @title: GdaMetaStruct
+ * @stability: Stable
+ * @see_also: #GdaMetaStore
+ *
+ * The #GdaMetaStruct object reads data from a #GdaMetaStore object and
+ *  creates an easy to use in memory representation for some database objects. For example one can easily
+ *  analyze the columns of a table (or its foreign keys) using a #GdaMetaStruct.
+ *
+ *  When created, the new #GdaMetaStruct object is empty (it does not have any information about any database object).
+ *  Information about database objects is computed upon request using the gda_meta_struct_complement() method. Information
+ *  about individual database objects is represented by #GdaMetaDbObject structures, which can be obtained using
+ *  gda_meta_struct_get_db_object() or gda_meta_struct_get_all_db_objects().
+ *
+ *  Note that the #GdaMetaDbObject structures may change or may be removed or replaced by others, so it not
+ *  advised to keep pointers to these structures: pointers to these structures should be considered valid
+ *  as long as gda_meta_struct_complement() and other similar functions have not been called.
+ *
+ *  In the following code sample, one prints the columns names and types of a table:
+ *  <programlisting>
+ *GdaMetaStruct *mstruct;
+ *GdaMetaDbObject *dbo;
+ *GValue *catalog, *schema, *name;
+ *
+ * // Define name (and optionnally catalog and schema)
+ *[...]
+ *
+ *mstruct = gda_meta_struct_new ();
+ *gda_meta_struct_complement (mstruct, store, GDA_META_DB_TABLE, catalog, schema, name, NULL);
+ *dbo = gda_meta_struct_get_db_object (mstruct, catalog, schema, name);
+ *if (!dbo)
+ *        g_print ("Table not found\n");
+ *else {
+ *        GSList *list;
+ *        for (list = GDA_META_TABLE (dbo)->columns; list; list = list->next) {
+ *                GdaMetaTableColumn *tcol = GDA_META_TABLE_COLUMN (list->data);
+ *                g_print ("COLUMN: %s (%s)\n", tcol->column_name, tcol->column_type);
+ *        }
+ *}
+ *gda_meta_struct_free (mstruct);
+ *  </programlisting>
+ *  If now the database object type is not known, one can use the following code:
+ *  <programlisting>
+ *GdaMetaStruct *mstruct;
+ *GdaMetaDbObject *dbo;
+ *GValue *catalog, *schema, *name;
+ *
+ * // Define name (and optionnally catalog and schema)
+ *[...]
+ *
+ *mstruct = gda_meta_struct_new ();
+ *gda_meta_struct_complement (mstruct, store, GDA_META_DB_UNKNOWN, catalog, schema, name, NULL);
+ *dbo = gda_meta_struct_get_db_object (mstruct, catalog, schema, name);
+ *if (!dbo)
+ *        g_print ("Object not found\n");
+ *else {
+ *        if ((dbo->obj_type == GDA_META_DB_TABLE) || (dbo->obj_type == GDA_META_DB_VIEW)) {
+ *                if (dbo->obj_type == GDA_META_DB_TABLE)
+ *                        g_print ("Is a table\n");
+ *                else if (dbo->obj_type == GDA_META_DB_VIEW) {
+ *                        g_print ("Is a view, definition is:\n");
+ *                        g_print ("%s\n", GDA_META_VIEW (dbo)->view_def);
+ *                }
+ *
+ *                GSList *list;
+ *                for (list = GDA_META_TABLE (dbo)->columns; list; list = list->next) {
+ *                        GdaMetaTableColumn *tcol = GDA_META_TABLE_COLUMN (list->data);
+ *                        g_print ("COLUMN: %s (%s)\n", tcol->column_name, tcol->column_type);
+ *                }
+ *        }
+ *        else 
+ *                g_print ("Not a table or a view\n");
+ *}
+ *gda_meta_struct_free (mstruct);
+ *  </programlisting>
+ */
 
 GType               gda_meta_struct_get_type           (void) G_GNUC_CONST;
 GdaMetaStruct      *gda_meta_struct_new                (GdaMetaStore *store, GdaMetaStructFeature features);
